#ifndef __KRTCCALLBACK_H__
#define __KRTCCALLBACK_H__

#include "KRtcTypeDef.h"

namespace KRTCSDK{

/// @defgroup KRtcCallback KRtcCallback类
/// 回调接口
/// @{
enum SDK_ERROR_NO : int32_t
{
    SDK_NO_ERROR,                                 ///< 成功
    SDK_NETWORK_ERROR,                            ///< 网络异常
    SDK_PARAMETER_ERROR,                          ///< 参数异常
    SDK_NOT_JOIN_ROOM_ERROR,                      ///< 未进入房间
    SDK_SERVER_ERROR,                             ///< 服务器异常
    SDK_LOGIN_REPEATED_REGISTRATION_ERROR,        ///< 登录重复注册   
    SDK_LOGIN_TOKEN_ERROR,                        ///< 登录Token错误
    SDK_JOINROOM_REPEATED_JOIN_ERROR,             ///< 重复入会
    SDK_JOINROOM_UNREGISTERED_ERROR,              ///< 未注册入会
    SDK_LEAVEROOM_PLEASE_OUT_ERROR,               ///< 被管理员请出
    SDK_LEAVEROOM_ROOM_DISSOLVED_ERROR,           ///< 房间被解散
    SDK_PUBLISHSTREAM_DEVICE_OPEN_ERROR,          ///< 设备开启失败
    SDK_PUBLISHSTREAM_NO_PERMISSIONS_ERROR,       ///< 无推双流权限
    SDK_PUBLISHSTREAM_CREATEPEERCONNECTION_ERROR, ///< 创建相互通道失败
    SDK_SUBSCRIBESTREAM_SPEAKER_ERROR,            ///< 扬声器异常
    SDK_SNAPSHOTVIDEO_OUTTIME_ERROR,              ///< 截图操作超时
    SDK_UNKNOWN_ERROR                             ///< 未知错误
};

class KRtcCallback
{
public:
    virtual ~KRtcCallback(){}

    /**
     * 警告回调
     *
     * @param warningType 信息类型
     * @param content     警告信息
     */
    virtual void onWarning(RTC_SDK_WARNING warningType, const char* content) = 0;

    /**
     * 错误回调
     *
     * @param warningType 错误类型
     * @param content     错误信息
     */
    virtual void onError(RTC_SDK_ERROR errorType, const char* content) = 0;

    /**
     * 资源释放回调
     *
     * @param result SDK_NO_ERROR：成功
     */
    virtual void onRelease(int32_t result) = 0;

    /**
     * 注册回调
     *
     * @param result SDK_NO_ERROR：成功 SDK_PARAMETER_ERROR：参数错误 SDK_LOGIN_REPEATED_REGISTRATION_ERROR：重复注册 SDK_NETWORK_ERROR：网络异常 SDK_LOGIN_TOKEN_ERROR：token校验失败
     * @param id 匿名登录时返回用户id
     */
    virtual void onLogin(int32_t result, userId id) = 0;

    /**
     * 登出回调
     *
     * @param result SDK_NO_ERROR：成功 SDK_NETWORK_ERROR：网络异常
     */
    virtual void onLogout(int32_t result) = 0;

    /**
     * 加入房间回调
     *
     * @param result SDK_NO_ERROR：成功 SDK_PARAMETER_ERROR：参数错误 SDK_JOINROOM_REPEATED_JOIN_ERROR：重复加入 SDK_JOINROOM_UNREGISTERED_ERROR：未注册平台 SDK_NETWORK_ERROR：网络异常
     */
    virtual void onJoinRoom(int32_t result) = 0;

    /**
     * 用户离开房间
     *
     * @param result，SDK_NO_ERROR：用户主动离开房间 SDK_NETWORK_ERROR：网络异常掉线 SDK_LEAVEROOM_PLEASE_OUT_ERROR：被管理员请出房间 SDK_LEAVEROOM_ROOM_DISSOLVED_ERROR：房间被解散
     */
    virtual void onLeaveRoom(int32_t result) = 0;

    /**
     * 收到平台呼叫
     *
     * @param roomId，房间id
     * @param roomId，房间名称
     *
     * @note 只有在未加入房间时会收到平台呼叫
     */
    virtual void onIncomingCall(const char* roomId, const char* roomName) {}

    /**
     * 平台取消呼叫
     *
     * @param roomId，房间id
     * @param roomId，房间名称
     */
    virtual void onCancelCall(const char* roomId, const char* roomName) {}

    /**
     * 响应平台呼叫回调
     *
     * @param answerOrHanup  true：应答 false：挂断
     * @param roomId         房间id
     * @param result         SDK_NO_ERROR：应答/挂断成功 SDK_PARAMETER_ERROR：参数错误 SDK_NETWORK_ERROR：网络异常
     */
    virtual void onReplyCall(bool answerOrHanup, const char* roomId, int32_t result) {}

    /**
     * 发布流回调
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引
     * @param result SDK_NO_ERROR：成功 SDK_PARAMETER_ERROR：参数错误 SDK_PUBLISHSTREAM_DEVICE_OPEN_ERROR：设备开启失败 SDK_SERVER_ERROR：服务器异常 SDK_NOT_JOIN_ROOM_ERROR：未进入房间 SDK_PUBLISHSTREAM_NO_PERMISSIONS_ERROR：无推双流权限 SDK_PUBLISHSTREAM_CREATEPEERCONNECTION_ERROR:创建通道失败
     */
    virtual void onPublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result) {}

    /**
     * 订阅远端用户流回调
     *
     * @param uid 用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引
     * @param result SDK_NO_ERROR：成功 SDK_NOT_JOIN_ROOM_ERROR：未进入房间 SDK_SERVER_ERROR：服务器异常 SDK_PARAMETER_ERROR：参数错误 SDK_SUBSCRIBESTREAM_SPEAKER_ERROR：扬声器开启失败
     */
    virtual void onSubscribeStream(userId uid, STREAM_TYPE type, uint8_t mediaIndex, int32_t result) {}

    /**
     * 设置优先层回调
     * @param uid 用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引
     * @param layerIndex  码流层数
     * @param result SDK_NO_ERROR：成功 SDK_SERVER_ERROR：服务器异常 SDK_PARAMETER_ERROR：参数错误 SDK_UNKNOWN_ERROR：未知错误
     */
    virtual void onUpdateStreamLayer(userId id, STREAM_TYPE type, uint8_t mediaIndex, uint8_t layerIndex, int32_t result) {}

    /**
     * 取消发布流回调
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引
     * @param result SDK_NO_ERROR：成功 SDK_UNKNOWN_ERROR：未知错误 
     */
    virtual void onUnpublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result) {}

    /**
     * 取消订阅流回调
     *
     * @param uid   用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引
     * @param result SDK_NO_ERROR：成功 SDK_SERVER_ERROR：服务器异常 SDK_PARAMETER_ERROR：参数错误
     */
    virtual void onUnsubscribeStream(userId uid, STREAM_TYPE type, uint8_t mediaIndex, int32_t result) {}

    /**
    * 取消发布流通知回调
    * @note sdk内部主动停止正在发布的流需通知上层
    *
    * @param type  码流类型
    * @param mediaIndex  码流索引，音频和辅流填0即可
    */
    virtual void onUnpublishStreamNotify(STREAM_TYPE type, uint8_t mediaIndex) {}

    /**
     * 暂停、恢复发布回调
     * @note 自定义采集或编码时不支持该功能
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频和辅流填0即可
     * @param pauseOrResume  true：暂停  false：恢复
     * @param result  SDK_NO_ERROR：成功 SDK_PARAMETER_ERROR：参数错误
     */
    virtual void onPausePublishStream(STREAM_TYPE type, uint8_t mediaIndex, bool pauseOrResume, int32_t result) {}

    /**
     * 暂停、恢复订阅回调
     *
     * @param id    远端用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频和辅流填0即可
     * @param pauseOrResume  true：暂停  false：恢复
     * @param result  SDK_NO_ERROR：成功 SDK_SERVER_ERROR：服务器异常 SDK_PARAMETER_ERROR：参数错误
     */
    virtual void onPauseSubscribeStream(userId id, STREAM_TYPE type, uint8_t mediaIndex, bool pauseOrResume, int32_t result) {}

    /**
     * 请求关键帧回调
     *
     * @param id    远端用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频和辅流填0即可
     * @param result  SDK_NO_ERROR：成功 SDK_PARAMETER_ERROR：参数错误 SDK_UNKNOWN_ERROR：未知错误 
     */
    virtual void onRequestKeyFrame(userId id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result) {}

    /**
     * 远端音频能量回调
     *
     * @param userVolumes  远端用户音频能量
     * @param speakerNumber userVolumes数组大小
     */
    virtual void onAudioLevelIndication(AudioLevelParam* userVolumes, uint32_t speakerNumber) {}

    /**
     * 远端用户加入房间
     *
     * @param id       远端用户id
     * @param nickName 远端用户别名
     */
    virtual void onRemoteUserJoinRoom(userId id, const char* nickName) = 0;

    /**
     * 远端用户离开房间
     *
     * @param id       远端用户id
     */
    virtual void onRemoteUserLeaveRoom(userId id) = 0;
    
    /**
     * 远端用户发布流
     *
     * @param id       远端用户id
     * @param param    流信息
     */
    virtual void onRemoteUserPublishStream(userId id, StreamParam* param) = 0;

    /**
     * 远端用户取消发布流
     *
     * @param id         远端用户id
     * @param type       码流类型
     * @param mediaIndex 同类型码流索引
     */
    virtual void onRemoteUserUnpublishStream(userId id, STREAM_TYPE type, uint8_t mediaIndex) = 0;

    /**
     * 设置编码参数
     * @note 仅在用户自定义编码时才会回调
     *
     * @param type       码流类型
     * @param mediaIndex 同类型码流索引
     * @param layerIndex simulcast分辨率层
     * @param param      编码参数
     */
    virtual void onRecommendEncodingParam(STREAM_TYPE type, uint8_t mediaIndex, uint8_t layerIndex, VideoCodecParam* param) {}

    /**
     * 本地网络统计信息回调，进入房间后2s触发一次
     *
     * @param state 网络统计信息
     */
    virtual void onLocalStatistics(Statistics* state) {}

    /**
     * 本地视频流统计信息回调，进入房间且发布后2s触发一次
     *
     * @param type       码流类型
     * @param mediaIndex 同类型码流索引
     * @param state 网络统计信息
     */
    virtual void onLocalVideoStatistics(STREAM_TYPE type, uint8_t mediaIndex, VideoStatistics* state) {}

    /**
     * 本地音频流统计信息回调，进入房间且发布后2s触发一次
     *
     * @param state 网络统计信息
     */
    virtual void onLocalAudioStatistics(AudioStatistics* state) {}

    /**
     * 远端视频流统计信息回调，进入房间后2s触发一次回调
     *
     * @param id         远端用户id
     * @param type       码流类型
     * @param mediaIndex 同类型码流索引
     * @param state 网络统计信息
     */
    virtual void onRemoteVideoStatistics(userId id, STREAM_TYPE type, uint8_t mediaIndex, VideoStatistics* state) {}

     /**
     * 远端音频流统计信息回调，进入房间后2s触发一次回调
     *
     * @param id         远端用户id
     * @param type       码流类型（用户音频或混音器音频）
     * @param state      网络统计信息
     */
    virtual void onRemoteAudioStatistics(userId id, STREAM_TYPE type, AudioStatistics* state) {}

    /**
     * 远端解码后视频帧回调
     *
     * @param id         远端用户id
     * @param type       码流类型
     * @param mediaIndex 同类型码流索引
     * @param data       帧数据
     */
    virtual void onRemoteVideoFrame(userId id, STREAM_TYPE type, uint8_t mediaIndex, VideoFrameData* data) {}

    /**
     * 远端解码前视频包回调
     *
     * @param id         远端用户id
     * @param type       码流类型
     * @param mediaIndex 同类型码流索引
     * @param data       解码前数据
     */
    virtual void onRemoteVideoData(userId id, STREAM_TYPE type, uint8_t mediaIndex, EncodedVideoData* data) {}

    /**
     * 混音后的音频解码数据回调
     *
     * @param data  音频数据
     */
    virtual void onRemoteMixedAudioFrame(AudioFrameData* data) {}

    /**
     * 远端解码后混音前的音频数据回调
     *
     * @param id  远端用户id
     * @param type  码流类型（用户音频、平台混音器音频）
     * @param data  音频数据
     */
    virtual void onRemoteAudioFrame(userId id, STREAM_TYPE type, AudioFrameData* data) {}

    /**
     * 本地采集音频数据回调（PCM格式）
     *
     * @param data  音频数据
     */
    virtual void onLocalAudioFrame(AudioFrameData* data) {}

    /**
      * 用于接收截图事件结果的上层接口，仅在未开启自定义采集的时候有效
      * @param id  远端用户id
      * @param type  码流类型（主流，辅流，画面合成）
      * @param mediaIndex  媒体通道id
      * @param rgb  转换的成的rgb格式的数据,默认为RGBA8888
      * @param result SDK_NO_ERROR:截图成功，SDK_SNAPSHOTVIDEO_OUTTIME_ERROR:截图操作超时
      */
    virtual void onSnapshotVideo(userId id, STREAM_TYPE type, uint8_t mediaIndex, const TRtcRawVideoData* rgb, int32_t result) {}

    /**
      * 用于接收静音事件的回调，终端或者平台控制时都会触发
      * @param playoutMute  true为静音
      */
    virtual void onAudioControlPlayoutMute(bool playoutMute) {}

    /**
      * 用于接收哑音事件的回调，终端或者平台控制时都会触发
      * @param recordMute  true为哑音
      */
    virtual void onAudioControlRecordMute(bool recordMute) {}
};
/// @}
}
#endif
