#ifndef __KRTCSDK_H__
#define __KRTCSDK_H__

#include "KRtcTypeDef.h"
#include "KRtcCallback.h"
#include "KRtcDeviceManager.h"

namespace KRTCSDK {

class KRtcClient;

/// @defgroup KRTCSDK KRTCSDK全局接口
/// 全局接口
/// @{
/**
 * 全局初始化
 *
 * @param callback   回调函数指针
 * @return  0：成功 -1：参数错误
 */
KDV_RTC_SDK_API int32_t init(KRtcSdkLogParam* param, KRtcSdkInitParam* initParam);

/**
 * 全局释放资源
 */
KDV_RTC_SDK_API void release();

/**
 * 创建实例
 *
 * @return  KRtcClient对象指针,返回空时创建失败
 */
KDV_RTC_SDK_API KRtcClient* createRtcInstance();

/**
 * 销毁实例（阻塞接口）
 *
 * @param rtcSdk   KRtcSdk对象指针
 * @return  0：成功 -1：参数错误
 */
KDV_RTC_SDK_API int32_t destroyRtcInstance(KRtcClient* rtcSdk);

/**
 * 获取DeviceManager单例对象
 *
 * @return  KRtcDeviceManager对象指针
 */
KDV_RTC_SDK_API KRtcDeviceManager* getDeviceManager();

/**
 * 获取DeviceManager单例对象
 *
 * @return  KRTCSDK版本号
 */
KDV_RTC_SDK_API const char* getVersion();
/// @}


/// @defgroup KRtcClient KRtcClient主要接口类
/// KRtcSdk主要接口类
/// @{
/**
* KRTCSDK主要接口类
*/
class KRtcClient
{
public:
    virtual ~KRtcClient(){}

    /**
     * 初始化实例
     *
     * @param callback   回调函数指针
     */
    virtual int32_t init(KRtcCallback* callback) = 0;

    /**
     * 登录平台
     *
     * @param param   注册参数
     */
    virtual void login(LoginParam* param) = 0;

    /**
     * 登出平台
     */
    virtual void logout() = 0;

    /**
     * 更新 Token
     *
     * @param token 新的 Token
     * @return      0：成功 -1：参数错误
     */
    virtual int32_t renewToken(const char* token) = 0;

    /**
     * 加入房间
     *
     * @param param   加入房间参数
     */
    virtual void joinRoom(JoinRoomParam* param) = 0;

    /**
     * 退出房间
     *
     */
    virtual void leaveRoom() = 0;

    /**
     * 响应平台呼叫
     *
     * @param answerOrHanup   true：应答 false：挂断
     * @param roomId          房间id
     */
    virtual void replyCall(bool answerOrHanup, const char* roomId) = 0;

    /**
     * 开启本地预览
     *
     * @param displayWindowId 回显窗口id
     *
     * @return 0：成功 -1：摄像头开启失败
     */
    virtual int32_t startLocalPreview(winId displayWindowId) = 0;

    /**
     * 停止本地预览
     *
     * @param deviceId        设备id，nullptr表示currentDevice
     * @param displayWindowId 回显窗口id
     */
    virtual void stopLocalPreview(winId displayWindowId) = 0;

    /**
     * 设置预览图像时的镜像模式
     *
     * @param mirror  true：镜像显示 false：取消镜像显示
     *
     * @return 0：成功 -1：参数错误
     */
    virtual int32_t setMirrorMode(winId displayWindowId, bool mirror) = 0;

    /**
     * 发布主视频流
     *
     * @param mediaIndex      发布多条主流时每个视频源的索引号
     */
    virtual void publishMainVideoStream(uint8_t mediaIndex) = 0;

#if _WIN32
    /**
     * 发布辅视频流
     *
     * @param captureScreenId  采集屏幕id
     * @param mediaIndex       发布多条辅流时每条流的索引号
     * @note 此接口只支持 Windows 平台
     */
    virtual void publishDualVideoStream(uint32_t captureScreenId) = 0;
#elif __ANDROID__

    /**
     * 发布辅视频流
     *
     * @note 此接口只支持 Android 平台
     */
    virtual void publishDualVideoStream() = 0;
#endif

    /**
     * 发布音频流
     */
    virtual void publishAudioStream() = 0;

    /**
     * 自定义主视频流参数
     *
     * @param param  发布主流参数
     * @note 需要在发布主流之前调用
     */
    virtual void setMainVideoStreamParam(MainVideoStreamParam* param) = 0;

    /**
     * 自定义辅视频流参数
     *
     * @param param  发布辅流参数
     * @note 需要在发布辅流之前调用
     */
    virtual void setDualVideoStreamParam(DualVideoStreamParam* param) = 0;

    /**
     * 订阅远端用户码流
     *
     * @param id              远端用户id，订阅合成器时无需此参数
     * @param param           订阅流参数
     * @param displayWindowId 本地回显窗口id，不需要则设0
     */
    virtual void subscribeStream(userId id, SubscribeStreamParam* param, winId displayWindowId) = 0;

    /**
     * 设置订阅码流的优选分辨率层
     * @note 仅支持主流订阅
     *
     * @param id   用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引
     * @param layerIndex  分辨率层
     */
    virtual void updateStreamLayer(userId id, STREAM_TYPE type, uint8_t mediaIndex, uint8_t layerIndex) = 0;

    /**
     * 取消发布码流
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频及辅流填0即可
     */
    virtual void unpublishStream(STREAM_TYPE type, uint8_t mediaIndex) = 0;

    /**
     * 取消订阅
     *
     * @param id         远端用户id
     * @param type       码流类型
     * @param mediaIndex 码流索引
     */
    virtual void unsubscribeStream(userId id, STREAM_TYPE type, uint8_t mediaIndex) = 0;

    /**
     * 设置暂停发布主流时的静态图像（暂停流时远端将显示静态图像，不设置则远端显示黑屏）
     *
     * @param image  图像缓存（BGRA格式）
     *
     * @return 0：成功 -1：参数错误
     */
    virtual int32_t setPauseImage(ImageBuffer* image) = 0;

    /**
     * 暂停、恢复发布（只影响本地）
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频和辅流填0即可
     * @param pauseOrResume  true：暂停  false：恢复
     */
    virtual void pausePublishStream(STREAM_TYPE type, uint8_t mediaIndex, bool pauseOrResume) = 0;

    /**
     * 暂停、恢复订阅（将通知平台暂停发送码流）
     *
     * @param id    远端用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频和辅流填0即可
     * @param pauseOrResume  true：暂停  false：恢复
     */
    virtual void pauseSubscribeStream(userId id, STREAM_TYPE type, uint8_t mediaIndex, bool pauseOrResume) = 0;

    /**
     * 手动请求关键帧
     *
     * @param id    远端用户id
     * @param type  码流类型
     * @param mediaIndex  码流索引
     */
    virtual void requestKeyFrame(userId id, STREAM_TYPE type, uint8_t mediaIndex) = 0;

    /**
     * 设置远端用户音频能量回调
     * @note 最小间隔为100ms，设0表示关闭回调，建议设置为300ms
     *
     * @param interval 音频能量回调间隔
     */
    virtual void enableAudioLevelIndication(int32_t interval) = 0;

    /**
     * 发送视频帧数据
     * @note 需要先调用 enableCustomVideoCapture 启用自定义视频采集
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引，辅流填0即可
     * @param data  帧数据（YUV420格式）
     *
     * @return 0：成功 -1：参数错误
     */
    virtual int32_t sendVideoFrameData(STREAM_TYPE type, uint8_t mediaIndex, VideoFrameData* data) = 0;

    /**
     * 发送编码后的视频数据
     * @note 需要先调用 enableCustomVideoEncode 启用自定义视频编码
     *
     * @param type  码流类型
     * @param mediaIndex  码流索引，音频和辅流填0即可
     * @param layerIndex  分辨率层
     * @param data  编码后的视频数据
     *
     * @return 0：成功 -1：参数错误
     */
    virtual int32_t sendEncodedVideoData(STREAM_TYPE type, uint8_t mediaIndex, uint8_t layerIndex, EncodedVideoData* data) = 0;

    /**
     * 发送音频数据
     * @note 需要先调用 enableCustomAudioCapture 启用自定义音频采集
     *
     * @param data  音频帧数据（PCM格式）
     *
     * @return 0：成功 -1：参数错误
     */
    virtual int32_t sendAudioFrameData(AudioFrameData* data) = 0;

    /**
     * 启用或关闭本地采集音频回调（PCM格式）
     *
     * @param enable true：开启 false：关闭
     * @param channelCount 通道数
     * @param sampleRate 音频采样率
     *
     */
    virtual void enableLocalAudioCallback(bool enable, uint32_t channelCount, uint32_t sampleRate) = 0;

    /**
     * 启用或关闭远端每一路音频数据回调（PCM格式）
     *
     * @param enable true：开启 false：关闭
     * @param channelCount 通道数
     * @param sampleRate 音频采样率
     *
     */
    virtual void enableRemoteAudioCallback(bool enable, uint32_t channelCount, uint32_t sampleRate) = 0;

    /**
     * 启用或关闭远端混音后音频数据回调（PCM格式）
     *
     * @param enable true：开启 false：关闭
     * @param channelCount 通道数
     * @param sampleRate 音频采样率
     *
     */
    virtual void enableRemoteMixedAudioCallback(bool enable, uint32_t channelCount, uint32_t sampleRate) = 0;

    /**
     * 设置音频设备能力
     *
     * @param micCapability 麦克风设备能力
     *
     * @note 仅启用自定义音频采集时需设置
     */
    virtual void setAudioDeviceCapability(AudioDeviceCapability* micCapability) = 0;


    /**
    * 设置截图事件的抽象接口
    * @param id  用户id,当用户id为""时表示为本地码流截图
    * @param type  码流类型（主流，辅流，画面合成）
    * @param mediaIndex  视频源索引号
    * @note 本地码流截图仅在未开启自定义采集的时候有效
    */
    virtual void snapshotVideo(userId id, STREAM_TYPE type, uint8_t mediaIndex) = 0;

    /**
    * 设置终端静音状态 
    * @param playoutMute  true为静音
    */
    virtual void setAudioControlPlayoutMute(bool playoutMute) = 0;

    /**
    * 设置终端哑音状态
    * @param playoutMute  true为哑音
    */
    virtual void setAudioControlRecordMute(bool recordMute) = 0;
};
/// @}
}

#endif
